/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;

/**
 * This class implements the interface {@link ICanControl} which represents a CAN
 * control unit and is used to control a CAN line.
 * 
 * @see ICanControl
 * @author Michael Ummenhofer
 * 
 */
public class CanControl extends CInterface implements ICanControl
{
  /** Maximums size of bus timing register table */
  private static final int CAN_BTR_TABLE_SIZE = 64;

  private static final int DetectBaud     = 3;
  private static final int InitLine       = 4;
  private static final int ResetLine      = 5;
  private static final int StartLine      = 6;
  private static final int StopLine       = 7;
  private static final int GetLineStatus  = 8;
  private static final int SetAccFilter   = 9;
  private static final int AddFilterIds   = 10;
  private static final int RemFilterIds   = 11;

  // Interface functions
  private CFunction m_fpDetectBaud      = null;
  private CFunction m_fpInitLine        = null;
  private CFunction m_fpResetLine       = null;
  private CFunction m_fpStartLine       = null;
  private CFunction m_fpStopLine        = null;
  private CFunction m_fpGetLineStatus   = null;
  private CFunction m_fpSetAccFilter    = null;
  private CFunction m_fpAddFilterIds    = null;
  private CFunction m_fpRemFilterIds    = null;

  // Interface Pointer
  private CPointer  m_ICanControl   = null; //Interface Pointer itself (pThis)

  /** CAN Line Status */
  private CanLineStatus m_oCanLineStatus = new CanLineStatus();

  //TODO: Is there an other way to create this object -> "protected"
  /**
   * Constructor
   * 
   * @param pICanControl
   * @throws Throwable
   */
  public CanControl(CMalloc pICanControl) throws Throwable
  {
    super(pICanControl);
    
    try
    {
      // Retrieve Interface Pointer Address
      m_ICanControl = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpDetectBaud      = new CFunction(m_ICanControl, DetectBaud,     "JNI");
      m_fpInitLine        = new CFunction(m_ICanControl, InitLine,       "JNI");
      m_fpResetLine       = new CFunction(m_ICanControl, ResetLine,      "JNI");
      m_fpStartLine       = new CFunction(m_ICanControl, StartLine,      "JNI");
      m_fpStopLine        = new CFunction(m_ICanControl, StopLine,       "JNI");
      m_fpGetLineStatus   = new CFunction(m_ICanControl, GetLineStatus,  "JNI");
      m_fpSetAccFilter    = new CFunction(m_ICanControl, SetAccFilter,   "JNI");
      m_fpAddFilterIds    = new CFunction(m_ICanControl, AddFilterIds,   "JNI");
      m_fpRemFilterIds    = new CFunction(m_ICanControl, RemFilterIds,   "JNI");
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_ICanControl        = null;
      m_fpDetectBaud       = null;
      m_fpInitLine         = null;
      m_fpResetLine        = null;
      m_fpStartLine        = null;
      m_fpStopLine         = null;
      m_fpGetLineStatus    = null;
      m_fpSetAccFilter     = null;
      m_fpAddFilterIds     = null;
      m_fpRemFilterIds     = null;
      
      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_ICanControl        = null;
      m_fpDetectBaud       = null;
      m_fpInitLine         = null;
      m_fpResetLine        = null;
      m_fpStartLine        = null;
      m_fpStopLine         = null;
      m_fpGetLineStatus    = null;
      m_fpSetAccFilter     = null;
      m_fpAddFilterIds     = null;
      m_fpRemFilterIds     = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }

  /**
   * @see ICanControl#DetectBaud(int, CanBitrate[])
   * @throws Throwable
   */
  public CanBitrate DetectBaud(int dwTimeout, CanBitrate[] aoBitrateTable) throws Throwable
  {
    CanBitrate  oCanBitrate = null;
    int         hResult     = 0;

    // Check Arguments
    if(aoBitrateTable.length > CAN_BTR_TABLE_SIZE)
      throw new IllegalArgumentException("CanBitrate[] has too many entries, only " + CAN_BTR_TABLE_SIZE + " are allowed.");

    // Create CAN InitLine structure Memory
    CMalloc pCanBtrTable = new CMalloc(130); //CANBTRTABLE

    try
    {
      // bCount - Number of values within the BTR tables
      pCanBtrTable.setByte(0, (byte)aoBitrateTable.length);
      
      // bIndex - Index of the selected value within the BTR table => Return Value
      pCanBtrTable.setByte(1, (byte)0);
      
      // Fill BaudRate List
      for(int i=0; i<aoBitrateTable.length; i++)
      {
        // abBtr0
        pCanBtrTable.setByte( 2+i, 
                              aoBitrateTable[i].get_bBtr0());
        // abBtr1
        pCanBtrTable.setByte( 2+i+CAN_BTR_TABLE_SIZE, 
                              aoBitrateTable[i].get_bBtr1());
      }

      // Detect BaudRate
      hResult = m_fpDetectBaud.callInt(new Object[]{ m_ICanControl, new Integer(dwTimeout), pCanBtrTable});
      if(hResult != 0)
      {
        throw new VciException("DetectBaud->InitLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        // bIndex - Index of the selected value within the BTR table => Return Value
        oCanBitrate = aoBitrateTable[pCanBtrTable.getByte(1)];
      }
    }
    finally
    {
      // Free Structure Memory
      pCanBtrTable.free();
    }
    
    return oCanBitrate;
  }

  
  /**
   * @see ICanControl#InitLine(int, CanBitrate)
   * @throws Throwable
   */
  public void InitLine(int dwOperatingMode, CanBitrate oCanBitrate) throws Throwable
  {
    int hResult = 0;

    // Create CAN InitLine structure Memory
    CMalloc pCanInitLine = new CMalloc(4);

    try
    {
      byte bOpMode  = (byte)dwOperatingMode;
      
      // Initialize CANINITLINE structure
      pCanInitLine.setByte(0, bOpMode);
      pCanInitLine.setByte(1, (byte)0); // bReserved
      pCanInitLine.setByte(2, oCanBitrate.get_bBtr0());
      pCanInitLine.setByte(3, oCanBitrate.get_bBtr1());

      // Initialize CAN Controller
      hResult = m_fpInitLine.callInt(new Object[]{ m_ICanControl, pCanInitLine});
      if(hResult != 0)
      {
        throw new VciException("ICanControl->InitLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
      // Free Structure Memory
      pCanInitLine.free();
    }
  }
  
  /**
   * @see ICanControl#ResetLine()
   * @throws Throwable
   */
  public void ResetLine() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpResetLine.callInt(new Object[]{ m_ICanControl });
      if(hResult != 0)
      {
        throw new VciException("ICanControl->ResetLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }
  
  /**
   * @see ICanControl#StartLine()
   * @throws Throwable
   */
  public void StartLine() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpStartLine.callInt(new Object[]{ m_ICanControl });
      if(hResult != 0)
      {
        throw new VciException("ICanControl->StartLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }
  
  /**
   * @see ICanControl#StopLine()
   * @throws Throwable
   */
  public void StopLine() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpStopLine.callInt(new Object[]{ m_ICanControl });
      if(hResult != 0)
      {
        throw new VciException("ICanControl->StopLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }

  /**
   * @see ICanControl#GetLineStatus()
   * @throws Throwable
   */
  public CanLineStatus GetLineStatus() throws Throwable
  {
    int hResult = 0;

    // Create CANLINESTATUS Memory
    CMalloc pCanLineStatus = new CMalloc(8);
    try
    {
      hResult = m_fpGetLineStatus.callInt(new Object[] { m_ICanControl, pCanLineStatus });
      if (hResult != 0)
      {
        throw new VciException("ICanControl->GetLineStatus failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oCanLineStatus.fromCPointer(pCanLineStatus);
      }
    } 
    finally
    {
      pCanLineStatus.free();
    }
    
    return m_oCanLineStatus;
  }
  
  /**
   * @see ICanControl#SetAccFilter(int, int, int)
   * @throws Throwable
   */
  public void SetAccFilter(int dwSelect, int dwCode, int dwMask) throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpSetAccFilter.callInt(new Object[] { m_ICanControl, new Integer(dwSelect), new Integer(dwCode), new Integer(dwMask) });
      if (hResult != 0)
      {
        throw new VciException("ICanControl->SetAccFilter failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }
   
  /**
   * @see ICanControl#AddFilterIds(int, int, int)
   * @throws Throwable
   */
  public void AddFilterIds(int dwSelect, int dwCode, int dwMask) throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpAddFilterIds.callInt(new Object[] { m_ICanControl, new Integer(dwSelect), new Integer(dwCode), new Integer(dwMask) });
      if (hResult != 0)
      {
        throw new VciException("ICanControl->AddFilterIds failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }
  
  /**
   * @see ICanControl#RemFilterIds(int, int, int)
   * @throws Throwable
   */
  public void RemFilterIds(int dwSelect, int dwCode, int dwMask) throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpRemFilterIds.callInt(new Object[] { m_ICanControl, new Integer(dwSelect), new Integer(dwCode), new Integer(dwMask) });
      if (hResult != 0)
      {
        throw new VciException("ICanControl->RemFilterIds failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }
   
  
  
}
