/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.jni.CPointer;;

/**
 * This value class represents the CAN Line Status.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class CanLineStatus
{
  /** transmission pending */
  private static final int CAN_STATUS_TXPEND = 0x01;
  /** data overrun occurred */
  private static final int CAN_STATUS_OVRRUN = 0x02;
  /** error warning limit exceeded */
  private static final int CAN_STATUS_ERRLIM = 0x04;
  /** bus off status */
  private static final int CAN_STATUS_BUSOFF = 0x08;
  /** init mode active */
  private static final int CAN_STATUS_ININIT = 0x10;

  /** current CAN operating mode */
  public byte m_bOpMode   = 0;  
  /** current bus timing register 0 value */
  public byte m_bBtReg0   = 0;  
  /** current bus timing register 1 value */
  public byte m_bBtReg1   = 0;  
  /** average bus load in percent (0..100) */
  public byte m_bBusLoad  = 0; 
  /** status of the CAN controller (see CAN_STATUS_) */
  private int m_dwStatus  = 0; 
  
  /**
   * transmission pending
   * <p>
   * For {@link #toString()}: <code>"P"</code> means transmission pending
   */
  public boolean m_fTxPending = false;
  
  /**
   * data overrun occurred
   * <p>
   * For {@link #toString()}: <code>"C"</code> means data overrun occurred
   */
  public boolean m_fRxOverrun = false;
  
  /**
   * error warning limit exceeded
   * <p>
   * For {@link #toString()}: <code>"W"</code> means error warning limit
   * exceeded
   */
  public boolean m_fWarningLevel = false;
  
  /**
   * bus off status
   * <p>
   * For {@link #toString()}: <code>"B"</code> means bus off status
   */
  public boolean m_fBusOff = false;
  
  /**
   * init mode active
   * <p>
   * For {@link #toString()}: <code>"I"</code> means init mode active
   */
  public boolean m_fCanInitMode = false;
  
  
  /**
   * Creates a new CAN Line Status object.
   * 
   * @throws Throwable
   */
  protected CanLineStatus()
  {
  }
  
  /**
   * Creates a new CAN Line Status object.
   * 
   * @param pCanLineStatus
   *          CPointer to native memory which contains the CANLINESTATUS
   *          structure.
   * @throws Throwable
   */
  protected CanLineStatus(CPointer pCanLineStatus) throws Throwable
  {
    fromCPointer(pCanLineStatus);
  }
  
  /**
   * Updates a existing CAN Line Status object.
   * 
   * @param pCanLineStatus
   *          CPointer to native memory which contains the CANLINESTATUS
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pCanLineStatus) throws Throwable
  {
    m_bOpMode   = pCanLineStatus.getByte(0);
    m_bBtReg0   = pCanLineStatus.getByte(1);
    m_bBtReg1   = pCanLineStatus.getByte(2);
    m_bBusLoad  = pCanLineStatus.getByte(3);
    m_dwStatus  = pCanLineStatus.getInt(4);
    
    if((m_dwStatus & CAN_STATUS_TXPEND) == CAN_STATUS_TXPEND)
      m_fTxPending = true;
    else
      m_fTxPending = false;
    
    if((m_dwStatus & CAN_STATUS_OVRRUN) == CAN_STATUS_OVRRUN)
      m_fRxOverrun = true;
    else
      m_fRxOverrun = false;
    
    if((m_dwStatus & CAN_STATUS_ERRLIM) == CAN_STATUS_ERRLIM)
      m_fWarningLevel = true;
    else
      m_fWarningLevel = false;
    
    if((m_dwStatus & CAN_STATUS_BUSOFF) == CAN_STATUS_BUSOFF)
      m_fBusOff = true;
    else
      m_fBusOff = false;
    
    if((m_dwStatus & CAN_STATUS_ININIT) == CAN_STATUS_ININIT)
      m_fCanInitMode = true;
    else
      m_fCanInitMode = false;
  }

  /**
   * Returns the CAN Line Status as string.
   * @return The CAN Line Status as string.
   */
  public String toString()
  {
    String strCanLineStatus = "";
    
    strCanLineStatus += "OpMode: 0x" + ZeroHex.toZeroHexString(m_bOpMode) + ", ";
    strCanLineStatus += "BtReg0: 0x" + ZeroHex.toZeroHexString(m_bBtReg0) + ", ";
    strCanLineStatus += "BtReg1: 0x" + ZeroHex.toZeroHexString(m_bBtReg1) + ", ";
    strCanLineStatus += "BusLoad: " + m_bBusLoad + "%, ";
    strCanLineStatus += "Flags: ";
    
    if(m_fTxPending)
      strCanLineStatus += "P";
    else
      strCanLineStatus += " ";
    
    if(m_fRxOverrun)
      strCanLineStatus += "C";
    else
      strCanLineStatus += " ";
    
    if(m_fWarningLevel)
      strCanLineStatus += "W";
    else
      strCanLineStatus += " ";
    
    if(m_fBusOff)
      strCanLineStatus += "B";
    else
      strCanLineStatus += " ";
    
    if(m_fCanInitMode)
      strCanLineStatus += "I";
    else
      strCanLineStatus += " ";
    
    return strCanLineStatus;
  }

}
