/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.vci3.*;
import de.ixxat.jni.*;

/**
 * This class implements the interface {@link ICanScheduler} which represents a
 * CAN scheduler.
 * 
 * @see ICanScheduler
 * @author Michael Ummenhofer
 * 
 */
public class CanScheduler extends CInterface implements ICanScheduler
{
  private static final int Resume       = 3;
  private static final int Suspend      = 4;
  private static final int Reset        = 5;
  private static final int GetStatus    = 6;
  private static final int AddMessage   = 7;
  private static final int RemMessage   = 8;
  private static final int StartMessage = 9;
  private static final int StopMessage  = 10;

  // Interface functions
  private CFunction m_fpResume          = null;
  private CFunction m_fpSuspend         = null;
  private CFunction m_fpReset           = null;
  private CFunction m_fpGetStatus       = null;
  private CFunction m_fpAddMessage      = null;
  private CFunction m_fpRemMessage      = null;
  private CFunction m_fpStartMessage    = null;
  private CFunction m_fpStopMessage     = null;

  // Interface Pointer
  private CPointer  m_ICanScheduler   = null; //Interface Pointer itself (pThis)

  /** CAN Scheduler Status */
  private CanSchedulerStatus m_oCanSchedulerStatus = new CanSchedulerStatus();

  //TODO: Is there an other way to create this object -> "protected"
  /**
   * Constructor
   */
  public CanScheduler(CMalloc pICanScheduler) throws Throwable
  {
    super(pICanScheduler);
    
    try
    {
      // Retrieve Interface Pointer Address
      m_ICanScheduler = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpResume          = new CFunction(m_ICanScheduler, Resume,        "JNI");
      m_fpSuspend         = new CFunction(m_ICanScheduler, Suspend,       "JNI");
      m_fpReset           = new CFunction(m_ICanScheduler, Reset,         "JNI");
      m_fpGetStatus       = new CFunction(m_ICanScheduler, GetStatus,     "JNI");
      m_fpAddMessage      = new CFunction(m_ICanScheduler, AddMessage,    "JNI");
      m_fpRemMessage      = new CFunction(m_ICanScheduler, RemMessage,    "JNI");
      m_fpStartMessage    = new CFunction(m_ICanScheduler, StartMessage,  "JNI");
      m_fpStopMessage     = new CFunction(m_ICanScheduler, StopMessage,   "JNI");
    }                                               
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_ICanScheduler     = null;
      m_fpResume          = null;
      m_fpSuspend         = null;
      m_fpReset           = null;
      m_fpGetStatus       = null;
      m_fpAddMessage      = null;
      m_fpRemMessage      = null;
      m_fpStartMessage    = null;
      m_fpStopMessage     = null;
      
      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_ICanScheduler     = null;
      m_fpResume          = null;
      m_fpSuspend         = null;
      m_fpReset           = null;
      m_fpGetStatus       = null;
      m_fpAddMessage      = null;
      m_fpRemMessage      = null;
      m_fpStartMessage    = null;
      m_fpStopMessage     = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * @see ICanScheduler#Resume()
   * @throws Throwable
   */
  public void Resume() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpResume.callInt(new Object[] { m_ICanScheduler });
      if (hResult != 0)
      {
        throw new VciException("ICanScheduler->Resume failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }
  
  /**
   * @see ICanScheduler#Suspend()
   * @throws Throwable
   */
  public void Suspend() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpSuspend.callInt(new Object[] { m_ICanScheduler });
      if (hResult != 0)
      {
        throw new VciException("ICanScheduler->Suspend failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }
  
  /**
   * @see ICanScheduler#Reset()
   * @throws Throwable
   */
  public void Reset() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpReset.callInt(new Object[] { m_ICanScheduler });
      if (hResult != 0)
      {
        throw new VciException("ICanScheduler->Reset failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }
  
  /**
   * @see ICanScheduler#GetStatus()
   * @throws Throwable
   */
  public CanSchedulerStatus GetStatus() throws Throwable
  {
    int hResult = 0;

    // Create CANSCHEDULERSTATUS Memory
    CMalloc pCanSchedulerStatus = new CMalloc(1 + 1*ICanScheduler.CAN_MAX_CTX_MSGS);
    try
    {
      hResult = m_fpGetStatus.callInt(new Object[] { m_ICanScheduler, pCanSchedulerStatus });
      if (hResult != 0)
      {
        throw new VciException("ICanScheduler->GetStatus failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oCanSchedulerStatus.fromCPointer(pCanSchedulerStatus);
      }
    } 
    finally
    {
      pCanSchedulerStatus.free();
    }
    
    return m_oCanSchedulerStatus;
  }

  /**
   * @see ICanScheduler#AddMessage(CanCyclicTxMessage, int)
   * @throws Throwable
   */
  public void AddMessage(CanCyclicTxMessage oCanCyclicTxMessage, int dwIndex) throws Throwable
  {
    int hResult = 0;

    // Create CANCYCLICTXMSG Memory
    CMalloc pCanCyclicTxMessage = new CMalloc(20);
    CMalloc pdwIndex = new CMalloc(4);
    try
    {
      // Update CPointer Memory
      oCanCyclicTxMessage.toCPointer(pCanCyclicTxMessage);
      pdwIndex.setInt(0, dwIndex);
      
      hResult = m_fpAddMessage.callInt(new Object[] { m_ICanScheduler,  pCanCyclicTxMessage, pdwIndex });
      if (hResult != 0)
      {
        throw new VciException("ICanScheduler->AddMessage failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
      pCanCyclicTxMessage.free();
      pdwIndex.free();
    }
  }

  /**
   * @see ICanScheduler#RemMessage(int)
   * @throws Throwable
   */
  public void RemMessage(int dwIndex) throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpRemMessage.callInt(new Object[] { m_ICanScheduler, new Integer(dwIndex) });
      if (hResult != 0)
      {
        throw new VciException("ICanScheduler->RemMessage failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }

  /**
   * @see ICanScheduler#StartMessage(int, int)
   * @throws Throwable
   */
  public void StartMessage(int dwIndex, int dwCount) throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpStartMessage.callInt(new Object[] { m_ICanScheduler, new Integer(dwIndex), new Integer(dwCount) });
      if (hResult != 0)
      {
        throw new VciException("ICanScheduler->StartMessage failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }

  /**
   * @see ICanScheduler#StopMessage(int)
   * @throws Throwable
   */
  public void StopMessage(int dwIndex) throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpStopMessage.callInt(new Object[] { m_ICanScheduler, new Integer(dwIndex) });
      if (hResult != 0)
      {
        throw new VciException("ICanScheduler->StopMessage failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }

}
