/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.vci3.IDisposable;

/**
 * This interface represents a CAN communication channel and is used to receive
 * and transmit CAN messages. When no longer needed the CAN communication
 * channel object has to be disposed using the {@link IDisposable} interface. A
 * CAN communication channel can be obtained via method
 * {@link ICanSocket#CreateChannel(boolean)}. Unless the CAN channel is not
 * already in use exclusively, it's possible to open several concurrently
 * communication channels on the same socket.
 * 
 * @author Michael Ummenhofer
 */
public interface ICanChannel extends IDisposable
{
  /**
   * This method initializes the CAN channel. This method must be called prior
   * to any other method of the interface.
   * <p>
   * <strong>Remarks:</strong> <blockquote> The channel is deactivated after
   * this method returns an must be activated by an {@link #Activate()} method
   * call. The method can be called more than once to reconfigure the size of
   * the receive or transmit FIFOs.</blockquote>
   * 
   * @param wRxSize
   *          Size of the receive buffer (number of CAN messages)
   * @param wTxSize
   *          Size of the transmit buffer (number of CAN messages)
   * @throws Throwable
   */
  public void Initialize(short wRxSize, short wTxSize) throws Throwable;

  /**
   * This method activates the CAN channel. After activating the channel, CAN
   * messages can be transmitted and received through the message writer and
   * message reader.
   * <p>
   * <strong>Remarks:</strong> <blockquote> The CAN line must be started,
   * otherwise no messages are received or transmitted from/to the CAN line (see
   * also {@link ICanControl#StartLine()}).</blockquote>
   * 
   * @throws Throwable
   */
  public void Activate() throws Throwable;

  /**
   * This method deactivates the CAN channel. After deactivating the channel, no
   * further CAN messages are transmitted or received to/from the CAN line.
   * 
   * @throws Throwable
   */
  public void Deactivate() throws Throwable;

  /**
   * Gets a reference to a new instance of a message reader object for the
   * channel. This message reader provides access to the channel's receive
   * buffer. CAN messages received from the CAN line can be read from this
   * object.
   * 
   * @return A reference to the message reader of the channel.
   * @throws Throwable
   */
  public ICanMessageReader GetMessageReader() throws Throwable;

  /**
   * Gets a reference to a new instance of a message writer object for the
   * channel. This message writer provides access to the channel's transmit
   * buffer. CAN messages written to this transmit buffer are transmitted over
   * the CAN line.
   * 
   * @return A reference to the message writer of the channel.
   * @throws Throwable
   */
  public ICanMessageWriter GetMessageWriter() throws Throwable;

  /**
   * Gets the current status of the CAN channel.
   * 
   * @return {@link CanChannelStatus}
   * @throws Throwable
   */
  public CanChannelStatus GetStatus() throws Throwable;
}
