/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.vci3.IDisposable;

/**
 * This interface represents a CAN message reader. When no longer needed the CAN
 * communication channel object has to be disposed using the {@link IDisposable}
 * interface. It's used to read received CAN messages from a CAN communication
 * channel (see {@link ICanChannel}). A CAN message reader object can be
 * obtained via the method {@link ICanChannel#GetMessageReader()}.
 * 
 * @author Michael Ummenhofer
 */
public interface ICanMessageReader extends IDisposable
{
  /**
   * Predefined timeout interval for {@link #WaitFor(int)}.
   * 
   * @see #WaitFor(int)
   */
  public static final int INFINITE = -1;

  /**
   * Gets the capacity of the receive FIFO in number of CAN messages.
   * 
   * @return Capacity
   * @throws Throwable
   */
  public int GetCapacity() throws Throwable;

  /**
   * Gets the number of currently unread CAN messages within the receive FIFO.
   * 
   * @return FillCount
   * @throws Throwable
   */
  public int GetFillCount() throws Throwable;

  /**
   * Gets the number of currently free CAN messages within the receive FIFO.
   * 
   * @return FreeCount
   * @throws Throwable
   */
  public int GetFreeCount() throws Throwable;

  /**
   * The WaitFor function returns when a new CAN message was received or the
   * time-out interval elapses. If the time-out value has elapsed or the waiting
   * was canceled by a call to {@link #CancelWait()} the function returns with
   * an exception.
   * 
   * @param dwMilliseconds
   *          Time-out interval, in milliseconds. The function returns if the
   *          interval elapses, even if no message was received. If
   *          dwMilliseconds is {@link #INFINITE}, the function's time-out
   *          interval never elapses.
   * @throws Throwable
   */
  public void WaitFor(int dwMilliseconds) throws Throwable;

  /**
   * Unblocks a thread waiting in {@link #WaitFor(int)}. If any thread was
   * waiting, the waiting thread will return with an exception.
   * 
   * @see #WaitFor(int)
   * 
   * @throws Throwable
   */
  public void CancelWait() throws Throwable;

  /**
   * This method reads a single CAN message from the front of the receive FIFO
   * and removes the message from the FIFO.
   * <p>
   * <strong>Remarks:</strong><blockquote>This method creates for every call a
   * new CanMessage object which has to be garbage collected by the runtime
   * environment later. Use {@link #ReadMessage(CanMessage)} instead.</blockquote>
   * 
   * @return CanMessage The CanMessage received.
   * @throws Throwable
   *           If the reception fails (e.g. the FIFO is empty) this function
   *           throws an exception.
   */
  public CanMessage ReadMessage() throws Throwable;

  /**
   * This method reads a single CAN message from the front of the receive FIFO
   * and removes the message from the FIFO.
   * <p>
   * <strong>Remarks:</strong><blockquote>This method is faster than
   * {@link #ReadMessage()}, but the user must provide the CanMessage Object
   * which members are updated for every new CanMessage received.</blockquote>
   * 
   * @param oCanMsg
   *          Reference to a CanMessage where the method stores the read
   *          message.
   * @return CanMessage The CanMessage received.
   * @throws Throwable
   *           If the reception fails (e.g. the FIFO is empty) this function
   *           throws an exception.
   */
  public CanMessage ReadMessage(CanMessage oCanMsg) throws Throwable;

}
